/*
 * Generic interrupt control functions for Broadcom MIPS boards
 *
 * Copyright (C) 2004 Broadcom Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <linux/autoconf.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/irq.h>

#include <asm/mipsregs.h>

#ifdef CONFIG_KGDB
#include <asm/gdb-stub.h>
#include <asm/kgdb.h>
#endif

#define ALLINTS (IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5)

extern asmlinkage void brcmIRQ(void);

/* void brcm_irq_dispatch(struct pt_regs *regs) */
asmlinkage void plat_irq_dispatch(void)
{
	u32 cause;

	cause = read_c0_cause() & read_c0_status() & CAUSEF_IP;

	clear_c0_status(cause);

	if (cause & CAUSEF_IP7)
		do_IRQ(7);
	if (cause & CAUSEF_IP2)
		do_IRQ(2);
	if (cause & CAUSEF_IP3)
		do_IRQ(3);
	if (cause & CAUSEF_IP4)
		do_IRQ(4);
	if (cause & CAUSEF_IP5)
		do_IRQ(5);
	if (cause & CAUSEF_IP6)
		do_IRQ(6);
}

static void enable_brcm_irq(unsigned int irq)
{
	if (irq < 8)
		set_c0_status(1 << (irq + 8));
	else
		set_c0_status(IE_IRQ0);
}

static void disable_brcm_irq(unsigned int irq)
{
	if (irq < 8)
		clear_c0_status(1 << (irq + 8));
	else
		clear_c0_status(IE_IRQ0);
}

static void ack_brcm_irq(unsigned int irq)
{
	/* Already done in brcm_irq_dispatch */
}

static unsigned int startup_brcm_irq(unsigned int irq)
{
	enable_brcm_irq(irq);

	return 0;		/* never anything pending */
}

static void end_brcm_irq(unsigned int irq)
{
	if (!(irq_desc[irq].status & (IRQ_DISABLED | IRQ_INPROGRESS)))
		enable_brcm_irq(irq);
}

/*
static struct hw_interrupt_type brcm_irq_type = {
      typename:"MIPS",
      startup:startup_brcm_irq,
      shutdown:disable_brcm_irq,
      enable:enable_brcm_irq,
      disable:disable_brcm_irq,
      ack:ack_brcm_irq,
      end:end_brcm_irq,
	NULL
};
*/

static struct irq_chip brcm_irq_chip = {
	name:"MIPS",
	ack:ack_brcm_irq,
	mask:disable_brcm_irq,
	mask_ack:ack_brcm_irq,
	unmask:enable_brcm_irq,
	end:end_brcm_irq,
	NULL
};

void __init arch_init_irq(void)
{
	int i;

	for (i = 0; i < NR_IRQS; i++) {
		irq_desc[i].status = IRQ_DISABLED;
		irq_desc[i].action = 0;
		irq_desc[i].depth = 1;
		/* irq_desc[i].handler = &brcm_irq_type; */
		set_irq_chip(i, &brcm_irq_chip);
	}

	/* set_except_vector(0, brcmIRQ);*/
	change_c0_status(ST0_IM, ALLINTS);
}
